@@ -17,7 +17,7 @@ import os, datetime
 import hashlib
 from collections import deque
 
-VERSION = "1.2.2"
+VERSION = "1.3.0"
 
 ###############################################################
 class idict(object):
@@ -263,7 +263,9 @@ class node(object):
     # Multiple paths from roots may exist. List of lists.
     self.paths = None
     # Lookup table for quick check if another node is in a path.
-    self.paths_set = set() 
+    self.paths_set = set()
+    # If false, will get file attributes.
+    self.phony = 0
     # None means TBD by graph walk. 
     # Otherwise 0 is uptodate, 1 means needs refresh.
     # If file doesn't exist, then update() will set stale to true.
@@ -332,12 +334,14 @@ class node(object):
     
   def update(self, time=True, hash=False):
     """Get modified time for file from system and optionally compute hash."""
-    self.stale = None
-    exists = os.path.exists(self.name)
+    self.reset()
     
+    if self.phony: return
+    
+    exists = os.path.exists(self.name)
     # Missing file is considered stale (we need to make it).
     if not exists: self.stale = 1
-    
+        
     if time and exists:
       self.time = os.path.getmtime(self.name)
 
@@ -388,6 +392,8 @@ class iterator(object):
     
 ###########################################################
 class dagger(object):
+  """Evaluates file dependencies in pure python."""
+  
   def __init__(self, hashfile='', sqlite=False, sqlite_memory=1):
     """
     Check if nodes are stale based on modified time or hash log (each line has file,hash).
@@ -408,6 +414,8 @@ class dagger(object):
     self.order = []
     # Nodes are only unique by name.
     self.nodes = {}
+    # Set of node names to consider phony.
+    self.phonys = set()
     # Use SQLite database instead of text file.
     self.sqlite = sqlite
     self.sqlite_memory = sqlite_memory
@@ -514,6 +522,10 @@ class dagger(object):
     if not no.paths: return no.paths
     return [[pno.name for pno in path] for path in no.paths]
 
+  def phony(self, name):
+    """Set node with name to be phony, so even when file with name is missing, it won't be considered stale, unless it has a stale child."""
+    self.phonys.add(name)
+    
   def resetnodes(self):
     """Reset all nodes."""
     [n.reset() for n in self.nodes.values()]
@@ -556,6 +568,7 @@ class dagger(object):
     # First pass get their latest info.
     for f in self.nodes:
       hash = self.hashall or self.useshash.get(f,0)
+      self.nodes[f].phony = f in self.phonys
       self.nodes[f].update( time=not hash, hash=hash )
     
     # Second pass, have each node check for a stale child,
@@ -598,19 +611,14 @@ class dagger(object):
               usehash = self.hashall or self.useshash.get(top.name,0)
               if usehash and self.db: dbhash = self.db.get(top.name)
               else: dbhash = None
-              # If db doesn't have hash for file, then its not stale.
-              if dbhash and dbhash <> top.hash:
+              # If db doesn't have hash for file because it wasn't hashed before, then its not stale yet.
+              if dbhash and (dbhash <> top.hash):
                 top.stale = 1
               else:
-                nOlder = 0
-                for child in top.nodes:
-                  if child.stale or \
-                    ( (not usehash) and (top.time < child.time) ):
-                    top.stale = 1
-                  else: nOlder +=1
-                
-                if nOlder == len(top.nodes):
-                  top.stale = 0
+                if (top.name in self.phonys) or usehash:
+                  top.stale = int( any([child.stale for child in top.nodes]) )
+                else: # Also compare with children time.
+                  top.stale = int( any([child.stale or (top.time < child.time) for child in top.nodes]) )
                   
           q.popleft()
           if top.paths <> None: # If not root node.
